@@ -254,7 +254,7 @@ EXCMD(CMD_caddbuffer,	"caddbuffer",	ex_cbuffer,
 	EX_RANGE|EX_WORD1|EX_TRLBAR,
 	ADDR_OTHER),
 EXCMD(CMD_caddexpr,	"caddexpr",	ex_cexpr,
-	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM|EX_TRLBAR,
+	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM,
 	ADDR_NONE),
 EXCMD(CMD_caddfile,	"caddfile",	ex_cfile,
 	EX_TRLBAR|EX_FILE1,
@@ -296,7 +296,7 @@ EXCMD(CMD_center,	"center",	ex_align,
 	EX_TRLBAR|EX_RANGE|EX_WHOLEFOLD|EX_EXTRA|EX_CMDWIN|EX_MODIFY,
 	ADDR_LINES),
 EXCMD(CMD_cexpr,	"cexpr",	ex_cexpr,
-	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM|EX_TRLBAR|EX_BANG,
+	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM|EX_BANG,
 	ADDR_NONE),
 EXCMD(CMD_cfile,	"cfile",	ex_cfile,
 	EX_TRLBAR|EX_FILE1|EX_BANG,
@@ -314,7 +314,7 @@ EXCMD(CMD_cgetbuffer,	"cgetbuffer",	ex_cbuffer,
 	EX_RANGE|EX_WORD1|EX_TRLBAR,
 	ADDR_OTHER),
 EXCMD(CMD_cgetexpr,	"cgetexpr",	ex_cexpr,
-	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM|EX_TRLBAR,
+	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM,
 	ADDR_NONE),
 EXCMD(CMD_chdir,	"chdir",	ex_cd,
 	EX_BANG|EX_FILE1|EX_TRLBAR|EX_CMDWIN,
@@ -407,8 +407,8 @@ EXCMD(CMD_cpfile,	"cpfile",	ex_cnext,
 	EX_RANGE|EX_COUNT|EX_TRLBAR|EX_BANG,
 	ADDR_OTHER),
 EXCMD(CMD_cquit,	"cquit",	ex_cquit,
-	EX_TRLBAR|EX_BANG,
-	ADDR_NONE),
+	EX_RANGE|EX_COUNT|EX_ZEROR|EX_TRLBAR|EX_BANG,
+	ADDR_UNSIGNED),
 EXCMD(CMD_crewind,	"crewind",	ex_cc,
 	EX_RANGE|EX_COUNT|EX_TRLBAR|EX_BANG,
 	ADDR_UNSIGNED),
@@ -442,6 +442,9 @@ EXCMD(CMD_debug,	"debug",	ex_debug,
 EXCMD(CMD_debuggreedy,	"debuggreedy",	ex_debuggreedy,
 	EX_RANGE|EX_ZEROR|EX_TRLBAR|EX_CMDWIN,
 	ADDR_OTHER),
+EXCMD(CMD_def,		"def",		ex_function,
+	EX_EXTRA|EX_BANG|EX_SBOXOK|EX_CMDWIN,
+	ADDR_NONE),
 EXCMD(CMD_delcommand,	"delcommand",	ex_delcommand,
 	EX_NEEDARG|EX_WORD1|EX_TRLBAR|EX_CMDWIN,
 	ADDR_NONE),
@@ -475,6 +478,9 @@ EXCMD(CMD_diffthis,	"diffthis",	ex_diffthis,
 EXCMD(CMD_digraphs,	"digraphs",	ex_digraphs,
 	EX_BANG|EX_EXTRA|EX_TRLBAR|EX_CMDWIN,
 	ADDR_NONE),
+EXCMD(CMD_disassemble,	"disassemble",	ex_disassemble,
+	EX_EXTRA|EX_NEEDARG|EX_TRLBAR|EX_CMDWIN,
+	ADDR_NONE),
 EXCMD(CMD_djump,	"djump",	ex_findpat,
 	EX_BANG|EX_RANGE|EX_DFLALL|EX_WHOLEFOLD|EX_EXTRA,
 	ADDR_LINES),
@@ -529,6 +535,9 @@ EXCMD(CMD_emenu,	"emenu",	ex_emenu,
 EXCMD(CMD_endif,	"endif",	ex_endif,
 	EX_TRLBAR|EX_SBOXOK|EX_CMDWIN,
 	ADDR_NONE),
+EXCMD(CMD_enddef,	"enddef",	ex_endfunction,
+	EX_TRLBAR|EX_CMDWIN,
+	ADDR_NONE),
 EXCMD(CMD_endfunction,	"endfunction",	ex_endfunction,
 	EX_TRLBAR|EX_CMDWIN,
 	ADDR_NONE),
@@ -544,6 +553,9 @@ EXCMD(CMD_endwhile,	"endwhile",	ex_endwhile,
 EXCMD(CMD_enew,		"enew",		ex_edit,
 	EX_BANG|EX_TRLBAR,
 	ADDR_NONE),
+EXCMD(CMD_eval,		"eval",		ex_eval,
+	EX_EXTRA|EX_NOTRLCOM|EX_SBOXOK|EX_CMDWIN,
+	ADDR_NONE),
 EXCMD(CMD_ex,		"ex",		ex_edit,
 	EX_BANG|EX_FILE1|EX_CMDARG|EX_ARGOPT|EX_TRLBAR,
 	ADDR_NONE),
@@ -553,6 +565,9 @@ EXCMD(CMD_execute,	"execute",	ex_execute,
 EXCMD(CMD_exit,		"exit",		ex_exit,
 	EX_RANGE|EX_WHOLEFOLD|EX_BANG|EX_FILE1|EX_ARGOPT|EX_DFLALL|EX_TRLBAR|EX_CMDWIN,
 	ADDR_LINES),
+EXCMD(CMD_export,	"export",	ex_export,
+	EX_EXTRA|EX_NOTRLCOM,
+	ADDR_NONE),
 EXCMD(CMD_exusage,	"exusage",	ex_exusage,
 	EX_TRLBAR,
 	ADDR_NONE),
@@ -569,7 +584,7 @@ EXCMD(CMD_filter,	"filter",	ex_wrongmodifier,
 	EX_BANG|EX_NEEDARG|EX_EXTRA|EX_NOTRLCOM,
 	ADDR_NONE),
 EXCMD(CMD_find,		"find",		ex_find,
-	EX_RANGE|EX_BANG|EX_FILE1|EX_CMDARG|EX_ARGOPT|EX_TRLBAR,
+	EX_RANGE|EX_BANG|EX_FILE1|EX_CMDARG|EX_ARGOPT|EX_TRLBAR|EX_NEEDARG,
 	ADDR_OTHER),
 EXCMD(CMD_finally,	"finally",	ex_finally,
 	EX_TRLBAR|EX_SBOXOK|EX_CMDWIN,
@@ -676,6 +691,9 @@ EXCMD(CMD_imapclear,	"imapclear",	ex_mapclear,
 EXCMD(CMD_imenu,	"imenu",	ex_menu,
 	EX_RANGE|EX_ZEROR|EX_EXTRA|EX_TRLBAR|EX_NOTRLCOM|EX_CTRLV|EX_CMDWIN,
 	ADDR_OTHER),
+EXCMD(CMD_import,	"import",	ex_import,
+	EX_EXTRA|EX_NOTRLCOM,
+	ADDR_NONE),
 EXCMD(CMD_inoremap,	"inoremap",	ex_map,
 	EX_EXTRA|EX_TRLBAR|EX_NOTRLCOM|EX_CTRLV|EX_CMDWIN,
 	ADDR_NONE),
@@ -743,7 +761,7 @@ EXCMD(CMD_language,	"language",	ex_language,
 	EX_EXTRA|EX_TRLBAR|EX_CMDWIN,
 	ADDR_NONE),
 EXCMD(CMD_laddexpr,	"laddexpr",	ex_cexpr,
-	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM|EX_TRLBAR,
+	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM,
 	ADDR_NONE),
 EXCMD(CMD_laddbuffer,	"laddbuffer",	ex_cbuffer,
 	EX_RANGE|EX_WORD1|EX_TRLBAR,
@@ -794,7 +812,7 @@ EXCMD(CMD_let,		"let",		ex_let,
 	EX_EXTRA|EX_NOTRLCOM|EX_SBOXOK|EX_CMDWIN,
 	ADDR_NONE),
 EXCMD(CMD_lexpr,	"lexpr",	ex_cexpr,
-	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM|EX_TRLBAR|EX_BANG,
+	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM|EX_BANG,
 	ADDR_NONE),
 EXCMD(CMD_lfile,	"lfile",	ex_cfile,
 	EX_TRLBAR|EX_FILE1|EX_BANG,
@@ -812,7 +830,7 @@ EXCMD(CMD_lgetbuffer,	"lgetbuffer",	ex_cbuffer,
 	EX_RANGE|EX_WORD1|EX_TRLBAR,
 	ADDR_OTHER),
 EXCMD(CMD_lgetexpr,	"lgetexpr",	ex_cexpr,
-	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM|EX_TRLBAR,
+	EX_NEEDARG|EX_WORD1|EX_NOTRLCOM,
 	ADDR_NONE),
 EXCMD(CMD_lgrep,	"lgrep",	ex_make,
 	EX_RANGE|EX_BANG|EX_NEEDARG|EX_EXTRA|EX_NOTRLCOM|EX_TRLBAR|EX_XFILE,
@@ -925,7 +943,7 @@ EXCMD(CMD_map,		"map",		ex_map,
 EXCMD(CMD_mapclear,	"mapclear",	ex_mapclear,
 	EX_EXTRA|EX_BANG|EX_TRLBAR|EX_CMDWIN,
 	ADDR_NONE),
-EXCMD(CMD_marks,	"marks",	do_marks,
+EXCMD(CMD_marks,	"marks",	ex_marks,
 	EX_EXTRA|EX_TRLBAR|EX_CMDWIN,
 	ADDR_NONE),
 EXCMD(CMD_match,	"match",	ex_match,
@@ -1304,19 +1322,19 @@ EXCMD(CMD_scscope,	"scscope",	ex_scscope,
 	EX_EXTRA|EX_NOTRLCOM,
 	ADDR_NONE),
 EXCMD(CMD_set,		"set",		ex_set,
-	EX_TRLBAR|EX_EXTRA|EX_CMDWIN|EX_SBOXOK,
+	EX_BANG|EX_TRLBAR|EX_EXTRA|EX_CMDWIN|EX_SBOXOK,
 	ADDR_NONE),
 EXCMD(CMD_setfiletype,	"setfiletype",	ex_setfiletype,
 	EX_TRLBAR|EX_EXTRA|EX_NEEDARG|EX_CMDWIN,
 	ADDR_NONE),
 EXCMD(CMD_setglobal,	"setglobal",	ex_set,
-	EX_TRLBAR|EX_EXTRA|EX_CMDWIN|EX_SBOXOK,
+	EX_BANG|EX_TRLBAR|EX_EXTRA|EX_CMDWIN|EX_SBOXOK,
 	ADDR_NONE),
 EXCMD(CMD_setlocal,	"setlocal",	ex_set,
-	EX_TRLBAR|EX_EXTRA|EX_CMDWIN|EX_SBOXOK,
+	EX_BANG|EX_TRLBAR|EX_EXTRA|EX_CMDWIN|EX_SBOXOK,
 	ADDR_NONE),
 EXCMD(CMD_sfind,	"sfind",	ex_splitview,
-	EX_BANG|EX_FILE1|EX_RANGE|EX_CMDARG|EX_ARGOPT|EX_TRLBAR,
+	EX_BANG|EX_FILE1|EX_RANGE|EX_CMDARG|EX_ARGOPT|EX_TRLBAR|EX_NEEDARG,
 	ADDR_OTHER),
 EXCMD(CMD_sfirst,	"sfirst",	ex_rewind,
 	EX_EXTRA|EX_BANG|EX_CMDARG|EX_ARGOPT|EX_TRLBAR,
@@ -1384,6 +1402,9 @@ EXCMD(CMD_spellinfo,	"spellinfo",	ex_spellinfo,
 EXCMD(CMD_spellrepall,	"spellrepall",	ex_spellrepall,
 	EX_TRLBAR,
 	ADDR_NONE),
+EXCMD(CMD_spellrare,	"spellrare",	ex_spell,
+	EX_BANG|EX_RANGE|EX_NEEDARG|EX_EXTRA|EX_TRLBAR,
+	ADDR_OTHER),
 EXCMD(CMD_spellundo,	"spellundo",	ex_spell,
 	EX_BANG|EX_RANGE|EX_NEEDARG|EX_EXTRA|EX_TRLBAR,
 	ADDR_OTHER),
@@ -1642,6 +1663,9 @@ EXCMD(CMD_vimgrep,	"vimgrep",	ex_vimgrep,
 EXCMD(CMD_vimgrepadd,	"vimgrepadd",	ex_vimgrep,
 	EX_RANGE|EX_BANG|EX_NEEDARG|EX_EXTRA|EX_NOTRLCOM|EX_TRLBAR|EX_XFILE,
 	ADDR_OTHER),
+EXCMD(CMD_vim9script,	"vim9script",	ex_vim9script,
+	0,
+	ADDR_NONE),
 EXCMD(CMD_viusage,	"viusage",	ex_viusage,
 	EX_TRLBAR,
 	ADDR_NONE),
@@ -1835,7 +1859,7 @@ struct exarg
     char_u	*(*getline)(int, void *, int, int);
     void	*cookie;	// argument for getline()
 #ifdef FEAT_EVAL
-    struct condstack *cstack;	// condition stack for ":if" etc.
+    cstack_T	*cstack;	// condition stack for ":if" etc.
 #endif
     long	verbose_save;	 // saved value of p_verbose
     int		save_msg_silent; // saved value of msg_silent
